//Static Shape Maps BETA 1.0
//by Wrapperup

//Video Tutorial and Tips to come.
//Allows modders to easily load static shapes.

//server.cs

datablock AudioProfile(glassBreakSound)
{
   filename    = "base/data/sound/glassBreak.wav";
   description = AudioDefault3d;
   preload = true;
};

datablock ParticleData(glassBreakParticle)
{
	dragCoefficient		= 0.0;
	windCoefficient		= 0.0;
	gravityCoefficient	= 2.0;
	inheritedVelFactor	= 0.0;
	constantAcceleration	= 0.0;
	lifetimeMS		= 10000;
	lifetimeVarianceMS	= 500;
	spinSpeed		= 10.0;
	spinRandomMin		= -50.0;
	spinRandomMax		= 50.0;
	useInvAlpha		= true;
	animateTexture		= false;
	//framesPerSec		= 1;

	textureName		= "base/data/particles/chunk";
	//animTexName		= "~/data/particles/cloud";

	// Interpolation variables
	colors[0]	= "1.0 1.0 1.0 1.0";
	colors[1]	= "0.0 0.0 0.0 0.0";
	sizes[0]	= 0.5;
	sizes[1]	= 0.5;
	times[0]	= 0.0;
	times[1]	= 9.0;
};

datablock ParticleEmitterData(glassBreakEmitter)
{
   ejectionPeriodMS = 1;
   periodVarianceMS = 0;
   lifetimeMS       = 10;
   ejectionVelocity = 15;
   velocityVariance = 10.0;
   ejectionOffset   = 0.0;
   thetaMin         = 0;
   thetaMax         = 90;
   phiReferenceVel  = 0;
   phiVariance      = 360;
   overrideAdvance = false;
   particles = "glassBreakParticle";

   useEmitterColors = true;
   uiName = "Spear Chunk";
   emitterNode = HalfEmitterNode;
};

$SSM::CurrentMap = -1;

function SSM_LoadMap(%file)
{
	if(isFile(%file) && getSubStr(%file,strLen(%file)-3,3) $= ".cs")
	{
		if(isObject(SSM_MapGroup))
		{
			SSM_MapGroup.chaindeletecallback = "SSM_LoadPhase(\"" @ %file @ "\");";
			SSM_MapGroup.chainDeleteAll();
		}
		else
			SSM_LoadPhase(%file);
	}
	else
	{
		error("SSM ERROR: File does not exist.");
	}
}

function SSM_LoadPhase(%file)
{
	if(isObject(SSM_MapGroup))
		SSM_MapGroup.delete();
		
	if(isFile(%file))
	{
		exec(%file);
		
		if(!isObject(SSM_MapGroup) || SSM_MapGroup.getClassName() !$= "ScriptGroup")
		{
			error("SSM ERROR: Static Shape load failed.");
		}
	}
	else
	{
		error("SSM ERROR: Static Shape load failed.");
	}
}

function SSM_addMapToList(%file, %name, %uiName, %pref)
{
	$SSM::Map[$SSM::MapCount] = %name;
	$SSM::MapUIName[$SSM::MapCount] = %uiname;
	
	$SSM::MapFile[$SSM::MapCount] = %file;
	$SSM::PrefFile[$SSM::MapCount] = %pref;
	
	$SSM::MapCount++;
}

function serverCmdSSM_requestMapList(%client)
{
	commandToClient(%client,'SSM_StartMapRecieve');
	
	if(%client.isSuperAdmin || %client.isLocalConnection())
		for(%i=0;%i<$SSM::MapCount;%i++)
		{
			commandToClient(%client,'SSM_RecieveMap',$SSM::Map[%i], $SSM::MapUIName[%i], %i);
		}
}

function SSM_RemoveMap()
{
	if(isObject(SSM_MapGroup))
	{
		SSM_MapGroup.chaindeletecallback = "";
		SSM_MapGroup.chainDeleteAll();
		PlayerDropPoints.getObject(0).setTransform("0 0 0");
		
		SSM_ResetPrefs();
	}
}

function serverCmdSSM_loadMap(%client,%a)
{		
	if(%client.isSuperAdmin || %client.isLocalConnection())
	{
		if(%a <= -1)
		{
			SSM_RemoveMap();
			return;
		}
		if(($Sim::Time - $SSM::Timeout) > 5)
		{
			SSM_LoadMap($SSM::MapFile[%a]);
			
			SSM_ParsePrefs($SSM::PrefFile[%a]);
		
			if($SSM::UseClock)
			{
				clockTick(1);
				setClock($Sim::Time);
			}
			
			$SSM::Timeout = $Sim::Time;
			$SSM::CurrentMap = %a;
		}
		else
		{
			%client.chatMessage("<color:ffffff>SSM: You are changing maps too quickly. Please wait 5 seconds.");
			commandToClient(%client,'SSM_setSelection',$SSM::CurrentMap);
		}
	}
}

function serverCmdSSM_Handshake(%client)
{
	%a = %client.isSuperAdmin;
	
	commandToClient(%client,'SSM_RecieveHandshake', %a);
}

function SSM_listMaps()
{
	for(%i=0;%i<$SSM::MapCount;%i++)
	{
		echo(%i SPC $SSM::Map[%i]);
	}
}

function serverCmdSSM_ReqCurrentMap(%client)
{
	if(%client.isSuperAdmin || %client.isLocalConnection())
		commandToClient(%client,'SSM_setSelection',$SSM::CurrentMap);
}

function SSM_clearMaps()
{
	$SSM::MapCount = 0;
}

function SSM_buildShapeList()
{
	%fileObj = new FileObject();
	
	SSM_clearMaps();
	
	%pattern = "Add-Ons/StaticMap_*/map.cs";
	for(%file=findFirstFile(%pattern);%file!$="";%file=findNextFile(%pattern))
	{
		%filePath = filePath(%file);
		
		if(isFile(%filePath @ "/description.txt"))
		{
			%pref = %filePath @ "/pref.txt";
			%folderName = fileBase(%filePath);
			%name = getSubStr(%folderName,10,strLen(%folderName));
			%uiName = strReplace(%name,"_"," ");
			
			SSM_addMapToList(%file, %name, %uiName, %pref);
		}
	}
}

if(!$SSM::ShapeListBuilt)
{
	$SSM::ShapeListBuilt = 1;
	SSM_buildShapeList();
}

function SSM_setSpawnSphere(%pos)
{
	PlayerDropPoints.getObject(0).setTransform(%pos);
}

function SSM_ParsePrefs(%filePath)
{
	if(!isFile(%filePath))
		return SSM_ResetPrefs();
	
	%file = new FileObject();
	%file.openForRead(%filePath);
	
	while(!%file.isEOF())
	{
		%line = %file.readLine();
		
		%var = getWord(%line,0);
		
		echo(%line);
		
		switch$(%var)
		{
			case "SpawnPosition":
				SSM_setSpawnSphere(restWords(%line));
				
			case "UseClock":
				$SSM::UseClock = restWords(%line);
		}
	}
}

function SSM_ResetPrefs()
{
	SSM_setSpawnSphere("0 0 0");
	$SSM::UseClock = 0;
}

function setClock(%time) //Helper function to set clock (enter in 1000, converts to a time string for you, and displays on the clock. Try setClock($Sim::Time);
{
	if(isObject(TenHours) && isObject(Hours) && isObject(TenMinutes) && isObject(Minutes))
	{
		%time = mFloor(%time);
		%time = getTimeString(%time);

		%time = strReplace(%time,":","	");
		
		if(getFieldCount(%time) == 0)
		{
			TenHours.playThread(0,"time0");
			Hours.playThread(0,"time0");
			
			TenMinutes.playThread(0,"time0");
			Minutes.playThread(0,"time0");
		}
		else if(getFieldCount(%time) <= 2)
		{
			%minutes = getField(%time,0);
			
			TenHours.playThread(0,"time0");
			Hours.playThread(0,"time0");
			
			if(strLen(%minutes) <= 1)
			{
				TenMinutes.playThread(0,"time0");
				Minutes.playThread(0,"time" @ getSubStr(%minutes,0,1));
			}
			else
			{
				TenMinutes.playThread(0,"time" @ getSubStr(%minutes,0,1));
				Minutes.playThread(0,"time" @ getSubStr(%minutes,1,2));
			}
		}
		else if(getFieldCount(%time) > 2)
		{
			%hours = getField(%time,0);
			%minutes = getField(%time,1);
			
			if(strLen(%hours) <= 1)
			{
				TenHours.playThread(0,"time0");
				Hours.playThread(0,"time" @ getSubStr(%hours,0,1));
			}
			else if(strLen(%hours) > 1)
			{
				TenHours.playThread(0,"time" @ getSubStr(%hours,0,1));
				Hours.playThread(0,"time" @ getSubStr(%hours,1,2));
			}
			
			if(strLen(%minutes) <= 1)
			{
				TenMinutes.playThread(0,"time0");
				Minutes.playThread(0,"time" @ getSubStr(%minutes,0,1));
			}
			else if(strLen(%minutes) > 1)
			{
				TenMinutes.playThread(0,"time" @ getSubStr(%minutes,0,1));
				Minutes.playThread(0,"time" @ getSubStr(%minutes,1,2));
			}
		}
	}
}

function clockTick(%new)
{
	if(isObject(TenHours) && isObject(Hours) && isObject(TenMinutes) && isObject(Minutes))
	{
		if(%new)
			while(isEventPending($SSM::Clock::Tick))
				cancel($SSM::Clock::Tick);
		
		$SSM::Clock::Tick = schedule(60000,0,clockTick);
		
		setClock($Sim::Time);
	}
}

package StaticMaps
{
	function Armor::onImpact(%this,%player,%col,%pos,%speed)
	{
		parent::onImpact(%this,%player,%col,%pos,%speed);

		if(%col.isBreakable)
		{

			ServerPlay3D(glassBreakSound,%player.getTransform());
			%emitter = new particleEmitterNode()
			{
				dataBlock = "GenericEmitterNode";
				emitter = "glassBreakEmitter";
				position = %col.getPosition();
			};
			
			%emitter.schedule(100,delete);
			
			%col.delete();
		}
	}
};

activatePackage(StaticMaps);
